#!/usr/bin/perl
#
##########################################################
# Televideo RAI Ultim'ora Retriever (tvdrai.pl)
# Copyright (C) 2001-2002 Paolo Cravero
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Version 1.1, 2002-01-01
$version = "1.1";
$version_date = "2002-01-01";
#
# Author: Paolo Cravero, ik1zyw at yahoo dot com 
#
##########################################################
#
# REQUIREMENTS
# - perl 5.x or higher
#   + perl IO::File module
# - lynx 2.8.4 or higher
# - internet connection :-)
#
##########################################################

use IO::File;

# START CONFIGURATION

# This is the base URL, the starting point for the news list
$baseURL = "http://www.televideo.rai.it/nazionale/homenaz.asp";

# When to invalidate a page if shorter than... bytes
$limitlength = 1000; #bytes

# How many levels should be retrieved (news are split in
# several pages with few news each, newest on top)
# Otherwisee the program stops when a news is already on disk
# whichever condition comes first
$level = 5; #must be higher or equal to 1

# Set the connection timeout to lynx.
# Lynx stops trying the page after $timeout seconds.
$timeout = 5; #seconds

# Set the break between page retrieval in seconds
$break = 0; #seconds

# The newsfeeds directory. Relative to tvdrai.pl home directory
# NO trailing slash!!
$newsdir = "./news";

# END CONFIGURATION

# Get the page with news list
sub get_home {

    my $i = shift;

    # run lynx and get a dump of the page
    # lynx's dump actually returns at the bottom the list of
    # URLs references in the page, so we grep only those that
    # have a NewsID
    print "Retrieving news list, page $i... ";
    $_ = `lynx -connect_timeout=$timeout -width=99999 -dump -force_html "$homeURL"   | grep NewsID`;

    # if the page is too short, there might have been a problem
    if (length($_)<$limitlength)
    {
	print "Error reading Televideo homepage ($homeURL)\nAbort...\n";
	exit 1;
    }
    print "done\n";
}


# Genereates a unique list of links to news pages
# Checks if the news is already on disk
sub split_news() {

    my $junk;
    my $url;
    my $pair;
    my $nn;

    # make an array out of the list of URLs provided by lynx
    @allclean = split(/\n/,$_);

    # insert news URLs into a hash
    foreach $pair (@allclean) {
	($junk, $url) = split(/\. /,$pair);

	# get the news ID out of the URL
	($junk, $nn) = split(/NewsID=/,$url);

	# if the news ID is already on disk, we proceed
	# with the retrieval of new feeds
	if ($nn == $latest_news_id) {
	    return 1;
	} else {
	    # otherwise we keep the link as good
	    $URL{$url}=1;
	}
    }

    return 0;

}


# Connect to the news server and retrieve a news page
sub get_news() {

    my $junk;

    # get the news entry ID and display it
    ($junk, $news_number) = split(/NewsID=/,$key);
    print "Downloading news number $news_number ... ";

    # we use lynx to get the page, the HTML source of it
    $_ = `lynx -connect_timeout=$timeout -source "$key"`;

    # if the page is too short, there might have been a problem
    # we just skip this news entry
    if (length($_)<$limitlength)
    {
	print "Error reading Televideo newspage ($key)\nSkipping...\n";
	return 0;
    }
    return 1;
}


# Save the news in the $newsdir directory
sub save_news() {

    my $junk;
    my $file;

    # then... save it
    $file = new IO::File "> $newsdir/$news_number.html";
    print $file $_;
    print "saved! (".--$newnews." left)\n";
    $file->close();
    
    return 0;
}


# Get the ID of the last news on disk
sub get_last() {

    # we list on a one column format (ls -1) the content of
    # news directory, and pick the last line.
    # entries are sorted by name, so the bottom line
    # is the newest we have downloaded
    $latest_news_id = `ls -1 $newsdir/*html | tail -1`;

    # lazy dirty way to obtain the ID number
    $latest_news_id =~ s/$newsdir\///g;
    $latest_news_id =~ s/.html\n//g;

    # if the directory is empty, set the ID to 0
    if (!$latest_news_id)  {$latest_news_id = 0;}

    return;
}


########
# MAIN #
########

print "\n      Televideo RAI Ultim'ora Retriever\n";
print "          Version $version ($version_date)\n";
print "            by Paolo Cravero (c)\n\n\n";

# get the ID of the latest news on disk
&get_last();

# and now proceed with identification of new news IDs
RETR: for (my $i=1;$i<=$level;$i++) {

    @allclean = ();

    # compose the URL for retrieving multiple pages
    $homeURL = $baseURL.+ "?currentStep=$i&Sezione=Ultimora\n";
    # get the page
    &get_home($i);

    # split the news into an array
    # exit the loop if we reach the existing news ID on disk
    if (&split_news()) {
	last RETR;
    }

}

# sort the news with oldest first
@keys = sort {$a cmp $b} keys %URL;

$newnews = scalar @keys;
print "There are ".$newnews." news to download.\n";

if (scalar @keys) {
    print "Starting to retrieve newsfeeds...\n";

    # retrieve all and only new news, that are stored in the %URL hash,
    # or, if you prefer, sorted in the @keys array
    foreach $key (@keys) {

        if (&get_news() ne 0) {
	    &save_news();
        }
    	sleep ($break); # wait before retrieving next page

    }

    print "\nAll new news downloaded.\nYou might want to launch \"lynx news/\" to read the news right away.\n";
}

print "Exiting...\n\n";
exit 0;

